/* $Id: filters.c,v 1.2 2007/02/21 15:42:57 tamentis Exp $
 *
 * Copyright (c) 2007 Bertrand Janin <tamentis@neopulsar.org>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR ONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "openqce.h"


void
filter_dump_raw(struct quickcam *qce)
{
	FILE *fp;

	fp = fopen("output.raw", "w");
	fwrite(qce->rawbuf, qce->width * qce->height, 1, fp);
	fclose(fp);
}



void
filter_render_grey(struct quickcam *qce)
{
	int i;

	for (i = 0; i < (qce->width * qce->height); i++) {
		qce->rgbframe[i * 3    ] = qce->rawbuf[i];
		qce->rgbframe[i * 3 + 1] = qce->rawbuf[i];
		qce->rgbframe[i * 3 + 2] = qce->rawbuf[i];
	}
}



void
filter_render_rgb(struct quickcam *qce)
{
	int x, y;
	unsigned long a;
	byte v;
	byte *f = qce->rgbframe;

	bzero(f, qce->width * qce->height * 3);

	for (y = 0; y < qce->height; y++) {
		for (x = 0; x < qce->width; x++) {
			a = y * (qce->width * 3) + x * 3;
			v = qce->rawbuf[y * qce->width + x];

			/* odd rows */
			if (y & 1) {
				if (x & 1)
					f[a + 1] = v;	// Green2
				else
					f[a] = v;	// Blue
			}
			/* even rows */
			else {
				if (x & 1)
					f[a + 2] = v;	// Red
				else
					f[a + 1] = v;	// Green1
			}
		}
	}

}



/* filter_render_2x2 - this create a picture with half the resolution */
void
filter_render_2x2(struct quickcam *qce)
{
	int x, y;
	unsigned long a, a2;
	byte r, g, b;
	byte *f = qce->rgbframe;

	for (y = 0; y < qce->height/2; y++) {
		for (x = 0; x < qce->width/2; x++) {
			a  =  y * 2      * (qce->width * 3) + x * 6;
			a2 = (y * 2 + 1) * (qce->width * 3) + x * 6;

			b = qce->rawbuf[y * 2 * qce->width + x * 2 + 1];
			r = qce->rawbuf[(y + 1) * 2 * qce->width + x * 2];
			g = (qce->rawbuf[y * 2 * qce->width + x * 2] +
			     qce->rawbuf[(y + 1) * 2 * qce->width + x * 2 + 1])
			    / 2;

			f[a    ] = f[a + 3] = f[a2    ] = f[a2 + 3] = r;
			f[a + 1] = f[a + 4] = f[a2 + 1] = f[a2 + 4] = g;
			f[a + 2] = f[a + 5] = f[a2 + 2] = f[a2 + 5] = b;
		}
	}

}



/* filter_render_full2x2 - same as above but do not cut the resolution by 
 * two, takes a little bit more time but still very fast. */
void
filter_render_full2x2(struct quickcam *qce)
{
	int x, y;
	unsigned long a;
	byte r, g, b;
	byte v00, v01, v10, v11;
	byte *f = qce->rgbframe;

	for (y = 0; y < qce->height; y++) {
		for (x = 0; x < qce->width; x++) {
			a = y * (qce->width * 3) + x * 3;
			v00 = qce->rawbuf[y * qce->width + x];
			v01 = qce->rawbuf[y * qce->width + x + 1];
			v10 = qce->rawbuf[(y + 1) * qce->width + x];
			v11 = qce->rawbuf[(y + 1) * qce->width + x + 1];

			/* odd rows */
			if (y & 1) {
				/* odd cols (green1) */
				if (x & 1) {
					g = (v00 + v11) / 2;
					b = v10;
					r = v01;
				/* even cols (red) */
				} else {
					r = v00;
					g = (v01 + v10) / 2;
					b = v11;
				}
			}

			/* even rows */
			else {
				/* odd cols (blue) */
				if (x & 1) {
					b = v00;
					g = (v01 + v10) / 2;
					r = v11;
				/* even cols (green2) */
				} else {
					g = (v00 + v11) / 2;
					b = v01;
					r = v10;
				}
			}

			f[a    ] = r;
			f[a + 1] = g;
			f[a + 2] = b;

		}
	}
}

