/* $Id: stv.c,v 1.2 2007/02/20 15:56:29 tamentis Exp $
 *
 * Copyright (c) 2007 Bertrand Janin <tamentis@neopulsar.org>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR ONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE. 
 */


#include <dev/usb/usb.h>
#include <sys/ioctl.h>

#include <stdio.h>
#include <string.h>

#include "openqce.h"


int
stv_get(struct quickcam *qce, int size, int reg, char *buffer)
{
	struct usb_ctl_request ur;
	int ret;

	ur.ucr_request.bmRequestType = 0xC0;
	ur.ucr_request.bRequest = 0x04;

	USETW(ur.ucr_request.wValue, reg); /* value */
	USETW(ur.ucr_request.wIndex, 0);  /* interface? */
	USETW(ur.ucr_request.wLength, size); /* length of data */

	ur.ucr_data = buffer;
	ur.ucr_flags = 0;
	ur.ucr_actlen = 0;

	memset(buffer, 0, size);

	if ((ret = ioctl(qce->ctrl_ep, USB_DO_REQUEST, &ur)) == -1) {
		perror("stv_get");
	}
	/* else {
		//printf("r 0x%04x: ", reg);
		for (i=0; i<size; i++) {
			//printf("0x%02x ", buffer[i]);
			if (i%8 == 7 || i == size-1) printf("\n");
		}
	}*/

	return (0);
}


char
stv_get_byte(struct quickcam *qce, int reg)
{
	char stuff[1];

	stv_get(qce, 1, reg, stuff);

	return (stuff[0]);
}


void
stv_get_word(struct quickcam *qce, int reg, int *buf)
{
	char stuff[2];

	stv_get(qce, 2, reg, stuff);

	*buf = stuff[0] + stuff[1] * (1 << 8);
}


int
stv_set(struct quickcam *qce, int size, int reg, unsigned char *buffer)
{
	struct usb_ctl_request ur;
	int ret;

	ur.ucr_request.bmRequestType = 0x40;
	ur.ucr_request.bRequest = 0x04;

	USETW(ur.ucr_request.wValue, reg); /* value */
	USETW(ur.ucr_request.wIndex, 0);  /* interface? */
	USETW(ur.ucr_request.wLength, size); /* length of data */

	ur.ucr_data = buffer;
	ur.ucr_flags = 0;
	ur.ucr_actlen = 0;

	if ((ret = ioctl(qce->ctrl_ep, USB_DO_REQUEST, &ur)) == -1) {
		perror("stv_set");
	}/* else {
		printf("w 0x%04x: ", reg);
		for (i=0; i<size; i++) {
			printf("0x%02x ", buffer[i]);
			if (i%8 == 7 || i == size-1) printf("\n");
		}
	}*/

	return (0);
}


void
stv_set_byte(struct quickcam *qce, int reg, char val)
{
	char stuff[1] = { val };

	stv_set(qce, 1, reg, stuff);
}


void
stv_dump(struct quickcam *qce)
{
	byte b;
	unsigned int i;

	printf("---------------------------------\n");
	printf(" STV0600-0 Registers\n");
	printf("---------------------------------\n\n");

	printf("0x0400: ");
	for (i = 0x0400; i < 0x410; i++) {
		b = stv_get_byte(qce, i);
		printf("%02X ", b);
	}
	printf("\n");
	printf("   ...\n");
	b = stv_get_byte(qce, 0x0420);
	printf("0x0420: %02X\n", b);

	printf("   ...\n");
	printf("0x1400: ");
	for (i = 0x1400; i < 0x1410; i++) {
		b = stv_get_byte(qce, i);
		printf("%02X ", b);
	}
	printf("\n");
	printf("   ...\n");
	b = stv_get_byte(qce, 0x1420);
	printf("0x1420: %02X\n", b);

	printf("   ...\n");
	b = stv_get_byte(qce, 0x1444);
	printf("0x1444: %02X ", b);
	b = stv_get_byte(qce, 0x1445);
	printf("%02X\n", b);

	printf("0x1500: ");
	for (i = 0x1500; i < 0x1505; i++) {
		b = stv_get_byte(qce, i);
		printf("%02X ", b);
	}
	printf("\n");

	printf("0x15C0: ");
	for (i = 0x15C0; i < 0x15C9; i++) {
		b = stv_get_byte(qce, i);
		printf("%02X ", b);
	}
	printf("\n");

	printf("   ...\n");
	b = stv_get_byte(qce, 0x1680);
	printf("0x1680: %02X\n", b);

}


void
stv_full_dump(struct quickcam *qce)
{
	unsigned int i;
	unsigned char b;

	for (i = 0; i < 0xFFFF; i++) {
		b = stv_get_byte(qce, i);
		printf("0x%04X: %02X\n", i, b);
	}
}

